(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function post(context) {
        return function (request, response, next) {
            if (request.method !== 'POST') {
                return next();
            }
            var executor = context.executor, handleMessage = context.handleMessage;
            try {
                var rawMessages = request.body;
                if (!Array.isArray(rawMessages)) {
                    rawMessages = [rawMessages];
                }
                var messages = rawMessages.map(function (messageString) {
                    return JSON.parse(messageString);
                });
                executor.log('Received HTTP messages');
                Promise.all(messages.map(function (message) { return handleMessage(message); }))
                    .then(function () {
                    response.statusCode = 204;
                    response.end();
                })
                    .catch(function () {
                    response.statusCode = 500;
                    response.end();
                });
            }
            catch (_) {
                response.statusCode = 500;
                response.end();
            }
        };
    }
    exports.default = post;
});
//# sourceMappingURL=post.js.map